## @class
#
package Statusmon::Statusmon;

# no warnings qw( uninitialized );

use Schedule;
use bbdisp;

@Statusmon::Statusmon::ISA = qw(Schedule);

%Statusmon::feature_providers = ();




## @fn init()
#
sub init {
    $Statusmon::display = new bbdisp( "$BigSister::common::fs{'adm'}/bb-display.cfg" );
    add_feature_provider( $Statusmon::display, "display" );
}


## @cmethod new()
#
sub new {
    print STDERR "new what??\n";
}


## @method setarg( $name, $val )
#
sub setarg {
    my( $self, $name, $val ) = @_;

    unless( defined $self->{"arguments"} ) {
	$self->{"arguments"} = {};
    }
    $self->{"arguments"}->{$name} = $val;
}



## @method getarg( $name )
#
sub getarg {
    my( $self, $name ) = @_;

    return( $self->{"arguments"}->{$name} );
}


## @fn start()
#
sub start {
    ;
}


## @method register( @masks )
# Register for a list of events.
#
# @param masks A list of event types to register for. If an event type is
#   prefixed with a '-' character, then the registration for this event type
#   is removed.
sub register {
    my( $self, @masks ) = @_;
    my $add = 1;

    unless( defined $self->{"reqmask"} ) {
	$self->{"reqmask"} = 0;
    }
    my $mask;
    foreach $mask (@masks) {
	if( substr( $mask, 0, 1 ) eq "+" ) {
	    $add = 1;
	    $mask = substr( $mask, 1 );
	}
	elsif( substr( $mask, 0, 1 ) eq "-" ) {
	    $add = 0;
	    $mask = substr( $mask, 1 );
	}
	$mask = $BigSister::common::cmd_groups{$mask};
	if( $add ) {
	    $self->{"reqmask"} |= $mask;
	}
	else {
	    $self->{"reqmask"} &= ~$mask;
	}
    }
}


## @method event( hashref cmd )
# Statusmon objects get their event() method invoked each time an event
# of one of the register() registered types is seen.
#
# param cmd The event.
sub event {
    my( $self, $cmd ) = @_;
    ;
}


## @method post_event( hashref cmd, hashref org_event )
# Post an event and forward it to all interested parties.
#
# @param cmd The event to post
# @param org_event (optional) If the event is posted as a reaction to another event
#   the original event must be passed in this argument.
sub post_event {
    my( $self, $cmd, $org_event ) = @_;

    if( $org_event ) {
        $cmd->{"id"} = $org_event->{"id"};
    }
    main::event( $cmd );
}


## @method add_feature_provider( $feature )
# Register as a feature provider.
#
# @param feature The name of the feature.
sub add_feature_provider {
    my( $self, $feature ) = @_;

    $Statusmon::feature_providers{$feature} = $self;
}


## @method Statusmon::Statusmon get_feature_provider( $feature )
# Get a Statusmon::Statusmon object that registered as a feature provider for a
# specific feature.
#
# @param feature The name of the feature.
# @return The registered object providing the feature, undef if none was found.
sub get_feature_provider {
    my( $self, $feature ) = @_;

    return(undef) unless( defined $Statusmon::feature_providers{$feature} );
    return( $Statusmon::feature_providers{$feature} );
}


## @method $ get_statustext( int code )
# Get the text associated with a status code.
#
# @param code The status code
# @return The status text (such as "red", "green", ...)
sub get_statustext {
    my( $self, $code ) = @_;

    return $BigSister::common::status_texts{$code};
}


## @method int get_statuscode( $text )
# For a given status text return the corresponding status code.
#
# @param text Status text.
# @return The status code that is associated with the given text.
sub get_statuscode {
    my( $self, $text ) = @_;

    return $BigSister::common::status_codes{$text};
}


## @method int get_current_status( $check )
# Returns the status of a check.
#
# @param check A string in the form "host.item" identifying a check.
# @return The status code, undef if no status was found for this check.
sub get_current_status {
    my( $self, $check ) = @_;

    if( defined $BigSister::common::current_status{$check} ) {
	return( $BigSister::common::current_status{$check}->{"status"} );
    }
    return undef;
}


## @method $ get_current_text( $check )
# Returns the current text of a check.
#
# @param check A string in the form "host.item" identifying a check.
# @return The text, undef if no status was found for this check.
sub get_current_text {
    my( $self, $check ) = @_;

    if( defined $BigSister::common::current_text{$check} ) {
	return( $BigSister::common::current_text{$check}->{"text"} );
    }
    return undef;
}



## @method @ get_checks( $group )
# Return a list of defined checks for a host or group.
#
# @param group A host or group name
# @return A list of check names defined for this host/group, an empty list if the
#   host/group is not known
sub get_checks {
    my( $self, $group ) = @_;

    if( defined $BigSister::common::current_status_c{$group} ) {
	return( keys %{$BigSister::common::current_status_c{$group}} );
    }
    return ();
}


## @method @ get_hosts()
# Returns a list of known hosts/groups.
#
# @return List of hosts/groups
sub get_hosts {
    my( $self ) = @_;

    return( keys %BigSister::common::current_status_c );
}


## @method abort()
# Stop this status monitor. This method is invoked by background() in order to
# signal the backgrounded process that it is not performing status monitoring any
# more.
sub abort {
}


## @method loopback( $cmd )
# Post a raw message and treat it like it was received from bbd.
#
# @param cmd The message string
sub loopback {
    my( $self, $cmd ) = @_;

    main::decode( $cmd );
}


## @method int background()
# Start a background process. Actually, this is more or less a frontend for
# fork(). In the child process, abort() is called.
#
# @return The pid of the child process, 0 in the child process, -1 if
#   backgrounding failed
sub background {
    my( $self ) = @_;

    if( ! $Platform::has_fork ) {
	return -1;
    }
    my $pid = fork();
    if( ($pid == -1) || ($pid > 0) ) { return $pid; }
    abort();
    return 0;
}

1;
