package RecordFile;


use strict;
use cfgfile;

sub new
{
    my ($this, $key, @files) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $self->init;
    $self->{cfgfile} = new cfgfile( @files );
    $self->{key} = $key;
    $self->{lastchecked} = 0;
    $self->{needread} = 1;
    return $self;
}


sub init {
    my( $self ) = @_;

    $self->{records} = [];
    $self->{bykey} = {};
}


sub add_file {
    my( $self, $file ) = @_;

    $self->{cfgfile}->add_file( $file );
}


sub check_files {
    my( $self, $time ) = @_;

    return if( (! $self->{needread}) && $time && (time-$self->{lastchecked} < $time) );
    $self->{lastchecked} = time;
    return unless( $self->{needread} || $self->{cfgfile}->has_changed() );
    $self->{needread} = 0;

    $self->init();
    foreach my $rec (@{$self->{cfgfile}->read_file()}) {
	$self->add_record( $rec );
    }
}


sub add_record {
    my( $self, $userdef ) = @_;
    
    my %rec = ();
    foreach my $def ( split( /\s+/, $userdef ) ) {
	if( $def =~ /^(.*?)=(.*)$/ ) {
	    $rec{$1} = $2;
	}
    }
    if( $self->{key} ) {
	return unless( defined $rec{$self->{key}} );
	$self->{bykey}->{$rec{$self->{key}}} = \%rec;
    }
    push( @{$self->{records}}, \%rec );
}



sub getall {
    my( $self ) = @_;

    $self->check_files(30);
    return( @{$self->{records}} );
}


sub get {
    my( $self, $key ) = @_;

    $self->check_files(30);
    return undef unless( defined $self->{bykey}->{$key} );
    return( $self->{bykey}->{$key} );
}



1;
