#!/usr/bin/perl
#    Big Sister network monitor
#    Copyright (C) 1998  Thomas Aeby
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#=============================================================================
#
$BigSister::common::Usage	  = "[-p pageid] file";
#
#=============================================================================
@BigSister::common::options = ( "p=i" );
use lib "."; use lib "uxmon"; use lib "platforms";	#inslib

use BigSister::common;
proginit();

use strict;

my $dl = $BigSister::common::dl;
our %image_cache;

use XML::Twig;

my $doc = new XML::Twig;
$doc->parsefile( $ARGV[0] );
$doc = $doc->root;

my $pageno = $BigSister::common::opt_p || "0";
my( $page ) = grep( $_->att("ID") eq $pageno, $doc->first_child("Pages")->children("Page") );

unless( $page ) {
    print STDERR "page '$pageno' missing\n";
    exit(1);
}

my $pageprops = $page->first_child( "PageSheet" )->first_child( "PageProps" );

my $pagewidth = width( $pageprops->first_child("PageWidth") );
my $pageheight = width( $pageprops->first_child("PageHeight") );

my $bground = $page->att("Name").".png";
unless( -f $bground ) {
    print STDERR "visio export $bground not found: please save your drawing as PNG as $bground\n";
    exit(1);
}
my( $imgx, $imgy ) = guess_size( $bground );
unless( $imgx && $imgy ) {
    print STDERR "image $bground: cannot determine its size\n";
    exit(1);
}

head();


my( $px, $py );

my( $bxl, $byt, $bxr, $byb );
foreach my $shape ( $page->first_child("Shapes")->children("Shape") ) {
    my $xform = $shape->first_child( "XForm" );
    next unless( $xform );
    my( $pinx, $piny, $locx, $locy, $width, $height ) = (
	width( $xform->first_child( "PinX" ) ),
	width( $xform->first_child( "PinY" ) ),
	width( $xform->first_child( "LocPinX" ) ),
	width( $xform->first_child( "LocPinY" ) ),
	width( $xform->first_child( "Width" ) ),
	width( $xform->first_child( "Height" ) ),
	);

    my $angle = $xform->first_child( "Angle" );
    # ignore rotated shapes
    next if( $angle && $angle->trimmed_text() );

    print STDERR "Shape ".($shape->att("ID"))." pinx=$pinx, piny=$piny, locx=$locx, locy=$locy, width=$width, height=$height\n" if( $BigSister::common::dl > 3 );
#    next if( $shape->att("NameU") =~ /Statusverbinder/ );
    $pinx -= $locx;
    $piny -= $locy;
    my $maxy = $piny + $height;
    my $maxx = $pinx + $width;

    if( (! defined($bxl)) || ($pinx < $bxl) ) { $bxl = $pinx; }
    if( (! defined($bxl)) || ($maxx < $bxl) ) { $bxl = $maxx; }
    if( (! defined($bxr)) || ($pinx > $bxr) ) { $bxr = $pinx; }
    if( (! defined($bxr)) || ($maxx > $bxr) ) { $bxr = $maxx; }

    if( (! defined($byb)) || ($piny < $byb) ) { $byb = $piny; print STDERR "byb: $byb (piny)\n" if( $dl > 2 ); }
    if( (! defined($byb)) || ($maxy < $byb) ) { $byb = $maxy;  print STDERR "byb: $byb (maxy)\n" if( $dl > 2 );}
    if( (! defined($byt)) || ($piny > $byt) ) { $byt = $piny;  print STDERR "byt: $byt (piny)\n" if( $dl > 2 );}
    if( (! defined($byt)) || ($maxy > $byt) ) { $byt = $maxy;  print STDERR "byt: $byt (maxy)\n" if( $dl > 2 );}
}
print STDERR "bounding box: left: $bxl, top: $byt, right: $bxr, bottom: $byb\n" if( $BigSister::common::dl > 1 );

foreach my $shape ( $page->first_child("Shapes")->children("Shape") ) {
    my $label;
    my( $mapping ) = grep( ($label=$_->first_child("Label")) && ($label->trimmed_text() eq "BigSister"), $shape->children("Prop") );
    next unless( $mapping && ($mapping->first_child("Value")) );
    my $mapent = $mapping->first_child("Value")->trimmed_text();
    next unless( $mapent );

    my $xform = $shape->first_child( "XForm" );
    next unless( $xform );
    $px = width( $xform->first_child("PinX") );
    $py = width( $xform->first_child("PinY") );
    return unless( defined($px) && defined($py) );


    my $geom = $shape->first_child( "Geom" );
    if( $geom && $geom->first_child( "LineTo" ) && $geom->first_child( "MoveTo" ) ) {
	# lines
	my( $x, $y ) = coords( $geom->first_child( "MoveTo" ) );
	my( $locx, $locy ) = (
	    width( $xform->first_child( "LocPinX" ) ),
	    width( $xform->first_child( "LocPinY" ) ),
	);
	$px -= $locx;
	$py -= $locy;

	foreach my $line ( $geom->children("LineTo") ) {
	    my( $nx, $ny ) = coords( $line );
	    if( defined($nx) ) {
		print "line ".tohcoords($x+$px,$y+$py)." ".tohcoords($nx+$px,$ny+$py)." $mapent\n";
		$x = $nx;
		$y = $ny;
	    }
	}
	next;
    }

    print "at ".tohcoords($px,$py)." ".$mapent."\n";
}

# returns width in cm
sub width {
    my( $node ) = @_;

    return( undef ) unless( $node );
    my $factor = 1;
    my $unit = $node->att("Unit");
    if( $unit eq "MM" ) {
	$factor = 1;	# Visio-MM seems to be cm
    }
    elsif( $unit eq "CM" ) {
	$factor = 1;
    }
    elsif( $unit eq "IN" ) {
	# not necessary: $factor = 2.54;
	$factor = 1;
    }
    elsif( $unit eq "IN_F" ) {
	# not necessary: $factor = 2.54;
	$factor = 1;
    }

    my $val = $factor * $node->trimmed_text();
    return $val;
}


sub coords {
    my( $node ) = @_;

    return(
	width( $node->first_child("X") ),
	width( $node->first_child("Y") )
    );
}




sub tohwidth {
    my( $size ) = @_;

    return( int($size/2.54*96) );
}


sub tohcoords {
    my( $x, $y ) = @_;

    $x -= $bxl;
    $x = int( 0.5 + $x/($bxr-$bxl)*$imgx );
    $y -= $byb;
    $y = $imgy - int( 0.5 + $y/($byt-$byb)*$imgy );
    return( "$x,$y" );
}

sub head {
    my( $x, $y ) = ($imgx,$imgy);
    my $bground = $page->att("Name");
    print <<END;
canvas         $x,$y
template	${bground}.png
red             skins/map/led_circle_red_flashing.gif
yellow          skins/map/led_circle_yellow.gif
green           skins/map/led_circle_green.gif
purple          skins/map/led_circle_purple.gif
blue            skins/map/led_circle_blue.gif
white           skins/map/led_circle_white.gif
clear           skins/map/led_circle_grey.gif

flash on
END
}



sub guess_size {
    my( $file ) = @_;

    my $path = $file;
    unless( -f $path ) {
	return undef;
    }
    my( $size, $mtime ) = (stat( $path ))[7,9];
    my $meta;
    if( defined( $image_cache{"$file $size $mtime"} ) ) {
	$meta = $image_cache{"$file $size $mtime"};
    }
    else {
	$meta = {};
	eval {
	    my $img = GD::Image->new($path);
	    my( $w, $h ) = $img->getBounds();
	    $meta->{width} = $w;
	    $meta->{height} = $h;
	};
	unless( $meta->{width} ) {
	    open( IMG, "-|", "identify", "-format", "%w:%h\\n", $path );
	    $_ = <IMG>;
	    chomp;
	    if( /^(\d+):(\d+)/ ) {
		$meta->{width} = $1;
		$meta->{height} = $2;
	    }
	    while( <IMG> ) {;}
	    close IMG;
	}
	unless( $meta->{width} ) {
	    $meta->{width} = 1;
	    $meta->{height} = 1;
	    BigSister::common::log( "warning", "unable to determine size of image $file: please install GD or ImageMagick" );
	}
    }
    print STDERR "size: $meta->{width} x $meta->{height}\n" if( $dl > 1 );
    $image_cache{"$file $size $mtime"} = $meta;
    return( $meta->{width}, $meta->{height} );
}


