#!/usr/bin/perl

package display_map_css;

use FileHandle;
use File::Copy;
use cfgfile;
use strict;

BEGIN {
    eval {
	require GD;
    };
}

our %image_cache = ();
our %param_cache = ();
our $refid = 0;
our %colors;


sub new
{
    my ($this, $display, $cfg) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $cfg = "display_map.cfg" unless( $cfg );
    $self->{"cfg"} = $cfg;
    $self->{"display"} = $display;
    print "display_map::new() created object for $cfg\n" if( $BigSister::common::dl>4 );
    return $self;
}


sub set_display {
    my( $self, $display ) = @_;

    $self->{"display"} = $display;
}


sub create {
    my ( $self, $view ) = @_;
    my $cfg = new cfgfile( $self->{"cfg"} );
    my $seq = FileHandle->new;
    my ($cmd, $args, @args);
    my( $image, %stats, $map, $i, %positions );
    my( $xw, $yw );

    $self->{wwwdir} = $BigSister::common::fs{web} . ($view ? "/$view" : "");
    my $html = "";
    $cfg->rewind();
    print "display_map::create() for ".($cfg->current_file())."\n" if( $BigSister::common::dl>4 );
    $self->{"map"} = $map = [];
    $colors{"red"} = "rgb(240,32,32)";
    $colors{"green"} = "rgb(0,208,0)";
    $colors{"yellow"} = "rgb(208,208,0)";
    $colors{"purple"} = "rgb(208,0,208)";
    $colors{"clear"} = "rgb(64,64,64)";
    $colors{"blue"} = "rgb(0,0,208)";
    $colors{"white"} = "rgb(255,255,255)";
    $colors{"black"} = "rgb(0,0,0)";

    while( $_ = $cfg->nextline() ) {
	print "display_map::read line: $_" if( $BigSister::common::dl>7 );
	chomp;
	next if( /^#/ || ! $_);
	if( /^(canvas|template|red|green|yellow|purple|clear|blue|white|graphics|at|dump|name|line|link|flash)[\s\t]+(.*)/ ) {
	    ($cmd, $args) = ($1,$2);
	    @args = split( /[\s\t]+/, $args );
	    if( $cmd eq "template" ) {
		print "display_map::create: read image from $args[0]\n" if( $BigSister::common::dl>5 );
		$image = $self->find_file( $args[0] );
		last unless( -f $self->{wwwdir}."/".$image );
		$html .= $self->image( $image, 0, 0, 0 );
		($xw,$yw) = $self->guess_size( $image ) unless( $xw && $yw );
	    }
	    elsif( $cmd eq "canvas" ) {
		my( $size ) = @args;
		( $xw, $yw ) = $self->coord( $size, %positions );
	    }
	    elsif( grep( $cmd eq $_, "red", "green", "purple", "yellow", "blue", "clear", "white" ) ) {
		$stats{$cmd} = $self->find_file( $args[0] );
		print "display_map::create: read status image $cmd from $args[0]\n" if( $BigSister::common::dl>5 );
	    }
	    elsif( $cmd eq "dump" ) {
		;
	    }
	    elsif( $cmd eq "link" ) {
		my( $pos, $target, $imagefile ) = @args;
		my( $x, $y ) = $self->coord( $pos, %positions );
		my( $w, $h, $ins );

		my $file = $self->find_file( $imagefile );
		my $img;
		if( $file ) {
		    $img = $self->image( $file, $x, $y, 1 );
		}
		else {
		    ($w,$h) = $self->coord( $imagefile, %positions );
		    $img .= "<div ". $self->style_pos( $x, $y, $w, $h, 1 ). "></div>";
		}
		$html .= $self->href( undef, $target, $img );
		#push( @$map, [ $x-$w/2, $y-$h/2, $w, $h, undef, $target ] );
		    
		print "display_map::create: inserted link $target at $pos\n" if( $BigSister::common::dl>5 );
	    }
	    elsif( $cmd eq "at" ) {
		my( $pos, $group, $ins, $text ) = @args;
		my( $x, $y ) = $self->coord( $pos, %positions );
		my( $w, $h, $status );

		my $status = $self->get_status( $group );
		my $extra = "";
		my $param = {};
		if( $ins =~ /^(\d+),(\d+)$/ ) {
		    ($w, $h) = ($1,$2);
		}
		elsif( $ins ) {
		    $param = $self->icon_param( $ins );
		    $ins = $self->find_file( $ins );
		}
		if( $w ) {
		    $extra = $self->background( $status );
		    if( defined($colors{$status})) {
			$extra .= "border-style:solid;border-width:1px;border-color:black;";
		    }
		    $extra .= "text-align:center;vertical-align:middle;line-height:${h}px;color:black;text-decoration:none;";
		    my $block = "<div ".$self->style_pos( $x, $y, $w, $h, 1, $extra ).">$text</div>\n";
		    if( defined($colors{$status})) {
			$html .= $self->href( $group, undef, $block );
		    }
		    else {
			$html .= $block;
		    }
		}
		elsif( $ins || ($ins = $stats{$status}) ) {
		    if( defined( $colors{$status} ) ) {
			if( defined( $param->{background} ) && $param->{background} ) {
			    my $bgstyle = "line-height:1px;font-size:1px;";
			    $bgstyle .= $self->background( $status );
			    if( $param->{bgborder} ) {
				$bgstyle .= "border-color:black;border-width:1px;border-style:solid;";
			    }
			    ( $w, $h ) = $self->coord( $param->{background} );
			    if( ($w<2) && ($h<2) ) {
				my( $xd, $yd ) = $self->guess_size( $ins );
				$w *= $xd;
				$h *= $yd;
			    }
			    $html .= "<div ".$self->style_pos( $x, $y, $w, $h, 1, $bgstyle )
				. "></div>";
			}
			elsif( $ins ne $stats{$status} && ! defined( $param->{background} )) {
			    $extra .= $self->background( $status );
			}
		    }
		    ($w,$h) = $self->guess_size( $ins );
		    $html .= $self->href( $group, undef, $self->image( $ins, $x, $y, 1, $extra ) );
		}
		if( $w && $h && defined($positions{$pos}) ) {
		    my $left = int( $x-$w/2 );
		    my $right = int( $left + $w );
		    my $top = int( $y-$h/2 );
		    my $bottom = int( $top + $h );
		    $positions{"$pos-topleft"} = [ $left, $top ];
		    $positions{"$pos-topright"} = [ $right, $top ];
		    $positions{"$pos-bottomleft"} = [ $left, $bottom ];
		    $positions{"$pos-bottomright"} = [ $right, $bottom ];
		    $positions{"$pos-bottom"} = [ $x, $bottom ];
		    $positions{"$pos-top"} = [ $x, $top ];
		}
		print "display_map::create: inserted status $status for $group at $pos\n" if( $BigSister::common::dl>5 );
	    }
	    elsif( $cmd eq "flash" ) {
		my( $onoff ) = @args;
		$self->{flashing} = ($onoff eq "off") ? 0:$onoff;
	    }
	    elsif( $cmd eq "name" ) {
		my( $pos, $name ) = @args;
		my( $x, $y ) = $self->coord( $pos, %positions );
		$positions{$name} = [$x,$y];
	    }
	    elsif( $cmd eq "line" ) {
		my( $pos1, $pos2, $group ) = @args;
		my( $status );

		$status = $self->get_status( $group );
		my @coord1 = $self->coord( $pos1, %positions );
		my @coord2 = $self->coord( $pos2, %positions );
		if( ($#coord1 == 1) && ($#coord2 == 1) ) {
		    my $color = $self->background( $status, "black" );
		    my @lines = ();
		    my $xd = $coord2[0] - $coord1[0];
		    my $yd = $coord2[1] - $coord1[1];
		    $color .= ";line-height:1px;font-size:1px;";
		    unless( $xd && $yd ) {
			push( @lines, "<div "
			    . $self->style_pos( @coord1, $xd ? $xd:2, $yd ? $yd:2, 0, $color )
			    . "></div>"
			);
		    }
		    else {
			my( $x1, $y1 ) = @coord1;
			my( $x2, $y2 ) = @coord2;
			my $ym = int($y1 + $yd/2);
			push( @lines, "<div "
			    . $self->style_pos( $x1, $y1, 2, $ym-$y1, 0, $color )
			    . "></div>" );
			push( @lines, "<div "
			    . $self->style_pos( $x1, $ym, $xd, 2, 0, $color )
			    . "></div>" );
			push( @lines, "<div "
			    . $self->style_pos( $x2, $ym, 2, $y2-$ym, 0, $color )
			    . "></div>" );
		    }

		    foreach my $line (@lines) {
			$html .= $self->href( $group, undef, $line );
		    }
		    print "display_map::create: lined status $status for $group\n" if( $BigSister::common::dl>5 );
		}
		else {
		    print "display_map::create: bad line coordinates for $group\n" if( $BigSister::common::dl>3 );
		}
	    }
	}
    }
    return( "<div style=\"position:relative;top:0px;left:0px;width:${xw}px;height:${yw}px;\">$html</div>" );
}

		

sub coord {
    my( $self, $pos, %positions ) = @_;

    if( defined $positions{$pos} ) {
	return( $positions{$pos}->[0], $positions{$pos}->[1] );
    }
    return( split( /[,;:,\.]/, $pos ) );
}



sub file {
    my( $self ) = @_;

    return $self->{"recent_file"};
}


sub map {
    my( $self ) = @_;

    return( $self->{"map"} ) if( $self->{"map"} );
    return( [] );
}


sub updated {
    my( $self ) = @_;
}



sub cfg {
    my( $self, $cfg ) = @_;

    $self->{"cfg"} = $cfg if( $cfg );
    return( $self->{"cfg"} );
}


sub get_status {
    my( $self, $group ) = @_;

    my( $host, $check ) = BigSister::common::separate_service( $group );
    my $status = "clear";
    if( $check ) {
	my $cols = $self->{"display"}->get_checks( $host );
	$status = $self->{"display"}->get_statustext( $cols->{$check} )
	  if( defined $cols->{$check} );
    }
    else {
	$status = $self->{"display"}->group_status( $group );
    }
    return $status;
}


sub icon_param {
    my( $self, $file ) = @_;

    if( defined( $param_cache{$file} ) ) {
	return $param_cache{$file};
    }
    my $path;
    if( -f $self->{wwwdir}."/".$file ) {
	$path = $self->{wwwdir}."/".$file;
    }
    else {
	$path = BigSister::common::find_file( $file );
    }
    $path =~ s/\.([^.]+)$/.par/;
    if( defined( $param_cache{$path} ) ) {
	return( $param_cache{$path} );
    }
    my $parms = {};
    if( -f $path ) {
	open( PAR, "<$path" );
	while( <PAR> ) {
	    chomp;
	    /^(.*?)=(.*)$/ and $parms->{$1} = $2;
	}
    }
    $param_cache{$path} = $param_cache{$file} = $parms;
}


sub find_file {
    my( $self, $file ) = @_;

    return $file if( -f $self->{wwwdir}."/".$file );
    $file = BigSister::common::find_file( $file );
    if( substr( $file, 0, length( $self->{wwwdir} ) ) eq $self->{wwwdir} ) {
	$file = substr( $file, length( $self->{wwwdir} ) );
	$file =~ s#^[/\\]*##;
	return $file;
    }
    return undef unless( -f $file );
    my $base = $file;
    $base =~ s#.*[/\\]##;
    my $dest = $self->{wwwdir}."/images/$base";
    ( -d $self->{wwwdir}."/images" ) || mkdir( $self->{wwwdir}."/images" );
    my @srcstat = stat( $file );
    my @dststat = stat( $dest );
    if( (!@dststat) || ($dststat[9] != $srcstat[9]) || ($dststat[7] != $srcstat[7]) ) {
	unless( copy( $file, $dest ) ) {
	    BigSister::common::log( "warning", "cannot install map image $file => $dest: $!" );
	}
    }
    return( "images/$base" );
}


sub href {
    my( $self, $refto, $ref, $element ) = @_;

    my $tref = "";
    if( $refto ) {
	$tref = "_MAPTITLE_${refid}_";
	$ref = "_MAPREFTO_". ($refid++) ."_";
	push( @{$self->{map}}, [ $refto, $ref, $tref ] );
    }
    return( "<a title=\"$tref\" href=\"$ref\">$element</a>" );
}


sub image {
    my( $self, $file, $x, $y, $middle, $extrastyle ) = @_;

    my( $w, $h ) = $self->guess_size( $file );
    my $name = $file;
    $name =~ s/\.[^.]+//;
    return( '<img border="0" src="'.$file.'" alt="'.$name.'" '
	. $self->style_pos( $x, $y, $w, $h, $middle, $extrastyle ). "/>" );
}


sub style_pos {
    my( $self, $x, $y, $w, $h, $middle, $extrastyle ) = @_;

    if( $h < 0 ) {
	$y += $h;
	$h = -$h;
    }
    if( $w < 0 ) {
	$x += $w;
	$w = -$w;
    }
    if( $middle ) {
	$x = int($x-$w/2);
	$y = int($y-$h/2);
    }
    return( "style=\"position:absolute"
	. ";top:" . $y
	. "px;left:". $x
	. "px;width:". $w
	. "px;height:". $h
	. "px;$extrastyle\"" 
    );
}


sub guess_size {
    my( $self, $file ) = @_;

    my $path = $self->{wwwdir}."/".$file;
    unless( -f $path ) {
	return undef;
    }
    my( $size, $mtime ) = (stat( $path ))[7,9];
    my $meta;
    if( defined( $image_cache{"$file $size $mtime"} ) ) {
	$meta = $image_cache{"$file $size $mtime"};
    }
    else {
	$meta = {};
	eval {
	    my $img = GD::Image->new($path);
	    my( $w, $h ) = $img->getBounds();
	    $meta->{width} = $w;
	    $meta->{height} = $h;
	};
	unless( $meta->{width} ) {
	    open( IMG, "-|", "identify", "-format", "%w:%h\\n", $path );
	    $_ = <IMG>;
	    chomp;
	    if( /^(\d+):(\d+)/ ) {
		$meta->{width} = $1;
		$meta->{height} = $2;
	    }
	    while( <IMG> ) {;}
	    close IMG;
	}
	unless( $meta->{width} ) {
	    $meta->{width} = 1;
	    $meta->{height} = 1;
	    BigSister::common::log( "warning", "unable to determine size of image $file: please install GD or ImageMagick" );
	}
    }
    $image_cache{"$file $size $mtime"} = $meta;
    return( $meta->{width}, $meta->{height} );
}


sub background {
    my( $self, $status, $defaultstatus ) = @_;

    unless( defined($colors{$status}) ) {
	return unless( $defaultstatus );
	$status = $defaultstatus;
    }
    my $style = "background-color:$colors{$status};";
    if( $self->{flashing} ) {
	my $img = $self->find_file( "skins/map/flash_$status.gif" );
	if( $img ) {
	    $style .= "background-image: url($img);";
	}
    }
    return $style;
}


    

1;
